//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.client;

import java.util.Collection;
import java.util.Iterator;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.ejb.ObjectNotFoundException;

import com.ibm.websphere.samples.bank.ejb.Customer;
import com.ibm.websphere.samples.bank.ejb.CustomerHome;
import com.ibm.websphere.samples.bank.ejb.CustomerKey;
import com.ibm.websphere.samples.bank.util.CustomerData;
import com.ibm.websphere.samples.bank.util.InvokeEJB;

public class FindAccounts {

    // Enterprise bean related
    private Customer customer = null;

    private final static String CUSTOMER_JNDI_NAME = "java:comp/env/ejb/Customer";

    /**
     * The main entrypoint to the TransferApplication when it is run as a Java application.
     * @param args java.lang.String[] The arguments to the Transfer application. There are none.
     */
    public static void main(String[] args) {
        try {
            if (args.length < 3) {
                System.out.println("You need to supply 3 parameters customerNumber, minBalance, lastName");
                System.exit(1);
            }
            System.out.println("Get a customer's account numbers exceeding a certain threshold\n");

            System.out.println("Getting the customer home...\n");
            Context ctx = new InitialContext();
            // Get the home object  MC
            Object homeObject = ctx.lookup(CUSTOMER_JNDI_NAME);
            // Get the CustomerHome 
            CustomerHome customerHome = (CustomerHome) javax.rmi.PortableRemoteObject.narrow(homeObject, CustomerHome.class);
            System.out.println("Done....\n");
            System.out.println("Finding the customer....");
            long custnum = Long.parseLong(args[0]);
            CustomerKey cKey = new CustomerKey(custnum);

            Customer customer = customerHome.findByPrimaryKey(cKey);

            System.out.println("Done....");

            System.out.println("Finding all the customer accounts sorted by account number...");
            float balance = Float.parseFloat(args[1]);
            Collection accountNumbers = customer.getAccountsByBalance(balance);
            if (accountNumbers != null) {
                Iterator it = accountNumbers.iterator();

                while (it.hasNext()) {
                    System.out.println("Account Number: " + ((Integer) it.next()).intValue());
                }
            } else {
                System.out.println("No accounts found with customer number " +custnum +" and balance " +balance);
            }
            System.out.println("All done! \n");


            System.out.println("Finding all the customers sorted by name...");
            Collection custcol = (new InvokeEJB()).findSortedCustomer(args[2]);

            if (custcol != null) {
                Iterator cit = custcol.iterator();

                while (cit.hasNext()) {
                    CustomerData cust = (CustomerData) cit.next();
                    System.out.println("Customer Name: " + cust.getFirst());
                    System.out.println("Customer taxid: " + cust.getId());
                }
            } else {
                System.out.println("No customers found with name: " +args[2]);
            }
            System.out.println("All done! \n");
            System.out.println("Done....");

            System.out.println("Finding all accounts for the customer exceeding balance " +balance +"...");
            Collection ccol = (new InvokeEJB()).getAccountsExceedBal(custnum, balance);

            Iterator ccit = ccol.iterator();

            while (ccit.hasNext()) {
                System.out.println("Account Number: " + ((Integer) ccit.next()).intValue());
            }
            System.out.println("All done! \n");
            System.out.println("Done....");

        } catch (ObjectNotFoundException e) {
            System.out.println("Customer number not found.");
        } catch (Exception e) {
            System.out.println("Exception caught! \n");
            e.printStackTrace();
        }

    }

}
