//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.client;

// general enterprise beans
import java.util.Collection;
import java.util.Iterator;

import javax.naming.Context;
import javax.naming.InitialContext;

import javax.ejb.ObjectNotFoundException;

import com.ibm.websphere.samples.bank.ejb.Customer;
import com.ibm.websphere.samples.bank.ejb.CustomerHome;
import com.ibm.websphere.samples.bank.ejb.CustomerKey;
import com.ibm.websphere.samples.bank.util.AccountData;
import com.ibm.websphere.samples.bank.util.InvokeEJB;

public class GetAccounts {

    // Enterprise bean related
    private Customer customer = null;

    private final static String CUSTOMER_JNDI_NAME = "java:comp/env/ejb/Customer";

    /**
     * The main entrypoint to the TransferApplication when it is run as a Java application.
     * @param args java.lang.String[] The arguments to the Transfer application. There are none.
     */
    public static void main(String[] args) {
        try {
            if (args.length < 1) {
                System.out.println("You need to supply 1 paramater: customerNumber");
                System.exit(1);
            }
            System.out.println("Get the account numbers owned by a certain customer\n");

            System.out.println("Getting the customer home...\n");
            Context ctx = new InitialContext();
            // Get the home object  MC
            Object homeObject = ctx.lookup(CUSTOMER_JNDI_NAME);
            // Get the CustomerHome 
            CustomerHome customerHome = (CustomerHome) javax.rmi.PortableRemoteObject.narrow(homeObject, CustomerHome.class);
            System.out.println("Done....\n");
            System.out.println("Finding the customer....");
            CustomerKey cKey = new CustomerKey(Long.parseLong(args[0]));

            Customer customer = customerHome.findByPrimaryKey(cKey);
            System.out.println("Done....");

            Collection accountNumbers = customer.getOwnedAccountNumbers();
            Iterator it = accountNumbers.iterator();

            while (it.hasNext()) {
                System.out.println("Account Number: " + ((Long) it.next()).longValue());
            }
            System.out.println("All done! \n");

            InvokeEJB cust = new InvokeEJB();
            System.out.println("Finding the customer from invoke ....");
            long custnm = Long.parseLong(args[0]);
            Collection accnum = cust.getOwnedAccountsNumber(custnm);
            if (accnum != null) {
                Iterator ait = accnum.iterator();

                while (ait.hasNext()) {
                    System.out.println("Account Number: " + ((Long) ait.next()).longValue());
                }
            }
            System.out.println("All done! \n");

            System.out.println("Get accounts owned by customer from invoke...");
            Collection accobj = cust.getOwnedAccounts(custnm);
            if (accobj != null) {
                Iterator cit = accobj.iterator();

                while (cit.hasNext()) {
                    AccountData act = (AccountData) cit.next();
                    System.out.println("Account Number: " + act.getAccNumber());
                    System.out.println("Account balance: " + act.getBalance());
                }
            }
            System.out.println("Get number of customer accounts from invoke...");
            int countacc = cust.getAccountsCount(custnm);
            System.out.println("Account :" + countacc);
            System.out.println("All done! \n");

        } catch (ObjectNotFoundException e) {
            System.out.println("Customer number not found.");
        } catch (Exception e) {
            System.out.println("Exception caught! \n");
            e.printStackTrace();
        }

    }

}
