/*==================================================================
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *===============================================================*/

package contactlist.forms;

import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import contactlist.struts.persistence.ContactListBroker;
import contactlist.struts.utilities.AIMException;


/**
 * Helper class to perform common functions for database actions
 * for contacts.
 * 
 * Creation date: (11/30/2003 10:00:00 PM)
 * @author: Tim Hanis
 */
public class ContactHelper {
	
	protected static ContactHelper contactHelper = null;
	protected Log log = null;
	
	/**
	 *  Return the singleton instance
	 */
	public static ContactHelper getInstance() {
		if (contactHelper == null) {
			contactHelper = new ContactHelper();
			contactHelper.log = LogFactory.getLog("com.ibm.sample.contacts.struts.beans");
		}
		return contactHelper;
	}
	

	/**
	 *  Create a new contact entry in the database
	 */
	public void addContact(ContactListBroker broker, Contact contact) 
		throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Adding contact for user "+contact.getOwner()+": "+contact);
		
		// Get a new oid and set it in contact bean
		Date date = new Date();
		String oid = ""+date.getTime();
		contact.setOid(oid);
		
		//  Add the contact to the database
		broker.saveContact(contact);
				
    }
    
    
	/**
	 *  Delete the contact
	 */
	public void deleteContact(ContactListBroker broker, String userid, String oid) 
		throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Deleting contact "+oid+" for user "+userid);
		
		broker.deleteContact(userid, oid);
		
    }


	/**
	 *  Get the contact object from session, delete the existing
	 *  entry and create a new contact entry
	 */
	public void modifyContact(ContactListBroker broker, Contact contact) 
			throws AIMException {
			
		//  Tracing
		if (log.isDebugEnabled())
			log.debug("Modifying contact "+contact.getOid()+" for user "+contact.getOwner());
		
		
		//  Delete the existing contact
		deleteContact(broker, contact.getOwner(), contact.getOid());
		
		//  Add the new entry
		addContact(broker, contact);		
		
    }

}

