package com.ibm.websphere.samples.bank.ejb;
import javax.ejb.FinderException;
import javax.ejb.EJBException;
import com.ibm.websphere.samples.bank.exception.InsufficientFundsException;
/**
 * Remote interface for Enterprise Bean: Transfer
 */
public interface Transfer extends javax.ejb.EJBObject {
	/**
	 * Returns the balance of the account identified by the acctId parameter.
	 * @return float The account balance.
	 * @param acctId long The account Id for which a balance is returned.
	 * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
	 * @exception javax.ejb.FinderException The specified account Id was not found.
	 */
	public float getBalance(int acctId)
		throws FinderException,
		EJBException,
		java.rmi.RemoteException;
	/**
	 * Transfers the specified amount from the fromAcctId account to the toAcctId account.
	 * @param fromAcctId long The account Id of the account from which the amount is transfered.
	 * @param toAcctId long The account Id of the account to which the amount is transfered.
	 * @param amount float The amount being transferred from the fromAcctId to the toAcctId.
	 * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
	 * @exception javax.ejb.FinderException The specified account Id was not found.
	 * @exception com.ibm.examples.bank.exception.InsufficientFundsException The account identified by
	 * the toAcctId contained less than the amount requested for transfer.
	 */
	public void transferFunds(int fromAcctId, int toAcctId, float amount)
		throws EJBException,
		InsufficientFundsException,
		FinderException,
		java.rmi.RemoteException;
}