//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

import java.io.IOException;
import java.io.Serializable;
import java.util.PropertyResourceBundle;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.CustomerKey;
import com.ibm.websphere.samples.bank.ejb.CustomerLocal;
import com.ibm.websphere.samples.bank.ejb.CustomerLocalHome;

import java.util.logging.*;

/**
 * The servlet creates a customer using the customer information from the 
 * jsp page.
 */

public class CreateCustomerServlet extends HttpServlet implements Serializable {

    // EJB access.
    private CustomerLocalHome customerHome = null;
    private static final String PROPFILE = "/WEB-INF/properties/bank_messages";
    PropertyResourceBundle bundle = (PropertyResourceBundle) PropertyResourceBundle.getBundle(PROPFILE);
    private final static String CUSTOMER_JNDI_NAME = "java:comp/env/ejb/Customer";

    static final String className = CreateCustomerServlet.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
    * Initialize the servlet
    */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);

        try {
            javax.naming.Context ctx = new javax.naming.InitialContext();
            customerHome = (CustomerLocalHome) ctx.lookup(CUSTOMER_JNDI_NAME);
        } catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errNamingInit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInitExpection") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }
    /**
    * doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);
    }

    /**
    * doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);

    }

    /**
     * @param req
     * @param resp
     * @throws ServletException
     */
    public void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        final String methodName = "performTask";

        try {
            PageBean pgBean = new PageBean();
            java.util.ArrayList msg = new java.util.ArrayList();
            java.util.ArrayList errmsg = new java.util.ArrayList();

            javax.servlet.http.HttpSession session = req.getSession(true);
            HistoryBean history = (HistoryBean) session.getAttribute("HistoryBean"); 
            if (history == null) {
      	        history = new HistoryBean(); // creates the bean if there isn't one on the session
            }

            String strCustNum = req.getParameter("customerNum");
            String firstName = req.getParameter("firstName");
            String lastName = req.getParameter("lastName");
            String taxID = req.getParameter("taxID");

            long longCustNum = 0;

            if (strCustNum == null || strCustNum.trim().length() == 0) {
                errmsg.add(bundle.getString("errCustNum"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCustNum"));
            } else {
                try {
                    pgBean.setCustomerNum(Long.parseLong(strCustNum));
                    longCustNum = Long.parseLong(strCustNum);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errCustNumFormat"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCustNumFormat"));
                }
            }

            if (firstName == null || firstName.trim().length() == 0) {
                errmsg.add(bundle.getString("errFirstName"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errFirstName"));
            } else {
                pgBean.setFirstName(firstName.toUpperCase());
            }

            if (lastName == null || lastName.trim().length() == 0) {
                errmsg.add(bundle.getString("errLastName"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errLastName"));
            } else {
                pgBean.setLastName(lastName.toUpperCase());
            }

            if (taxID == null || taxID.trim().length() == 0) {
                errmsg.add(bundle.getString("errTaxID"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errTaxID"));
            } else {
                pgBean.setTaxID(taxID.toUpperCase());
            }

            // Datatypes used to create new Customer
            CustomerKey key;

            //After page validation if there are no errors 
            if (errmsg.size() == 0) {

                try {
                    firstName = firstName.toUpperCase();
                    lastName = lastName.toUpperCase();
                    taxID = taxID.toUpperCase();
                    int count = 0;

                    CustomerKey ckey = new CustomerKey(longCustNum);
                    CustomerLocal cust = customerHome.create(ckey, firstName, lastName, taxID);

                    //adding the data to the bean for page display
                    CustomerData data = new CustomerData(longCustNum, firstName, lastName, count, taxID);

                    req.setAttribute("CustData", data);

                    //message
                    msg.add(bundle.getString("succNewCustomer"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("succNewCustomer"));

                    history.addMessage(bundle.getString("succNewCustomer")); // adds a message to the command history
                } catch (DuplicateKeyException e) {
                    errmsg.add(bundle.getString("errDuplicateCustomer"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errDuplicateCustomer") + e.getMessage());
                } catch (CreateException e) {
                    errmsg.add(bundle.getString("errCreateCustomer"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCreateCustomer") + e.getMessage());
                    e.printStackTrace();
                } catch (Exception e) {
                    errmsg.add(bundle.getString("errGenFailure"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errGenFailure") + e.getMessage());
                    e.printStackTrace();
                }
            } //end of if

            pgBean.setMessageLines(msg);
            pgBean.setErrormessage(errmsg);
            req.setAttribute("PageBean", pgBean);
            session.setAttribute("HistoryBean", history);// stores it back into the session

            if (pgBean.getErrormessage().size() > 0) {
                getServletContext().getRequestDispatcher("/jsp/createcustomer.jsp").forward(req, resp);
            } else {
                getServletContext().getRequestDispatcher("/jsp/customersuccess.jsp").forward(req, resp);
            }
        } catch (Throwable ex) {
            throw new ServletException(ex);
        }
    }
}
