//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

import java.io.IOException;
import java.io.Serializable;
import java.util.PropertyResourceBundle;
import java.text.MessageFormat;

import javax.ejb.FinderException;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.TransferLocal;
import com.ibm.websphere.samples.bank.ejb.TransferLocalHome;

import java.util.logging.*;

/**
 * This servlet helps to find the balance of a particular account.	
 * 
 */
public class GetBalanceServlet extends HttpServlet implements Serializable {

    // EJB access.
    private TransferLocalHome transferHome = null;
    private static final String PROPFILE = "/WEB-INF/properties/bank_messages";
    PropertyResourceBundle bundle = (PropertyResourceBundle) PropertyResourceBundle.getBundle(PROPFILE);
    private final static String TRANSFER_JNDI_NAME = "java:comp/env/ejb/Transfer";

    static final String className = GetBalanceServlet.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
    * Initialize the servlet
    */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";

        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);

        try {
            javax.naming.Context ctx = new javax.naming.InitialContext();
            transferHome = (TransferLocalHome) ctx.lookup(TRANSFER_JNDI_NAME);
        } catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errNamingInit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInitExpection") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }

    }
    /**
    * doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);
    }

    /**
    * doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);

    }

    /**
     * @param req
     * @param resp
     * @throws ServletException
     */
    public void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        final String methodName = "performTask";

        try {
            PageBean pgBean = new PageBean();
            java.util.ArrayList msg = new java.util.ArrayList();
            java.util.ArrayList errmsg = new java.util.ArrayList();

            java.text.DecimalFormat dF = new java.text.DecimalFormat("####.00");

            javax.servlet.http.HttpSession session = req.getSession(true);
            HistoryBean history = (HistoryBean) session.getAttribute("HistoryBean"); 
            if (history == null) {
      	        history = new HistoryBean(); // creates the bean if there isn't one on the session
            }

            String strAcctNum = req.getParameter("accountNum");

            int intAcctNum = 0;
            String balance;

            if (strAcctNum == null || strAcctNum.trim().length() == 0) {
                errmsg.add(bundle.getString("errAcctNum"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctNum"));
            } else {
                try {
                    pgBean.setAccountNum(Integer.parseInt(strAcctNum));
                    intAcctNum = Integer.parseInt(strAcctNum);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errAcctValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctValid"));
                }
            }

            //After page validation if there are no errors 
            if (errmsg.size() == 0) {
                try {

                    // Create a Transfer object.
                    TransferLocal transfer = transferHome.create();
                    balance = dF.format(transfer.getBalance(intAcctNum));

                    Object[] msgArgs = new Object[2];
                    msgArgs[0] = strAcctNum;
                    msgArgs[1] = balance;
                    MessageFormat msgFormat = new MessageFormat(bundle.getString("succAcctBalance"));
                    String msgAcctBalance = msgFormat.format(msgArgs);
                    msg.add(msgAcctBalance);
                    messageLogger.logp(Level.INFO, className, methodName, msgAcctBalance);                                       

                    history.addMessage(msgAcctBalance); // adds a message to the command history
                } catch (FinderException e) {
                    errmsg.add(bundle.getString("errFindBalance"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errFindBalance"));
                } catch (Exception e) {
                    errmsg.add(bundle.getString("errExpBalance"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errExpBalance"));
                    e.printStackTrace();
                }
            } //end of if

            pgBean.setMessageLines(msg);
            pgBean.setErrormessage(errmsg);
            req.setAttribute("PageBean", pgBean);
            session.setAttribute("HistoryBean", history);// stores it back into the session

            getServletContext().getRequestDispatcher("/jsp/getbalance.jsp").forward(req, resp);

        } catch (Throwable ex) {
            throw new ServletException(ex);
        }
    }

    /**
     * @param x
     * @return String respresentation of a float
     */
    private String formatFloat(float x) {
        String s = null;
        if (((x * 100) % 10) == 0) {
            s = x + "0";
        } else {
            s = String.valueOf(x);
        }
        return s;
    }
}
