//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

import java.io.IOException;
import java.io.Serializable;
import java.util.PropertyResourceBundle;

import javax.ejb.FinderException;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.CustomerKey;
import com.ibm.websphere.samples.bank.ejb.CustomerLocal;
import com.ibm.websphere.samples.bank.ejb.CustomerLocalHome;

import java.util.logging.*;

/**
 * This servlet searches for a list of customers using the customers last 
 * name.
 */
public class SearchByCustomerServlet extends HttpServlet implements Serializable {

    // EJB access.
    private CustomerLocalHome customerHome = null;
    private static final String PROPFILE = "/WEB-INF/properties/bank_messages";
    PropertyResourceBundle bundle = (PropertyResourceBundle) PropertyResourceBundle.getBundle(PROPFILE);
    private final static String CUSTOMER_JNDI_NAME = "java:comp/env/ejb/Customer";

    static final String className = SearchByCustomerServlet.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
    * Initialize the servlet
    */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);

        try {
            javax.naming.Context ctx = new javax.naming.InitialContext();
            customerHome = (CustomerLocalHome) ctx.lookup(CUSTOMER_JNDI_NAME);

        } catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errNamingInit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInitExpection") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }

    /**
    * doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);
    }

    /**
    * doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);

    }

    /**
     * @param req
     * @param resp
     * @throws ServletException
     */
    public void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        final String methodName = "performTask";

        try {
            PageBean pgBean = new PageBean();
            java.util.ArrayList msg = new java.util.ArrayList();
            java.util.ArrayList errmsg = new java.util.ArrayList();

            javax.servlet.http.HttpSession session = req.getSession(true);
            HistoryBean history = (HistoryBean) session.getAttribute("HistoryBean"); 
            if (history == null) {
      	        history = new HistoryBean(); // creates the bean if there isn't one on the session
            }

            String lastName = req.getParameter("lastName");

            if (lastName == null || lastName.trim().length() == 0) {
                errmsg.add(bundle.getString("errLastNameReq"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errLastNameReq"));
            }

            //Customer sorted by name
            java.util.ArrayList list = new java.util.ArrayList();

            if (errmsg.size() == 0) {

                try {
                    java.util.Collection col = customerHome.findByLastName(lastName.toUpperCase() + "%");
                    java.util.Iterator it = col.iterator();

                    messageLogger.logp(Level.INFO, className, methodName, "The method findbylastName() returned " + col.size() + " rows");

                    while (it.hasNext()) {
                        CustomerLocal c = (CustomerLocal) it.next();

                        messageLogger.logp(Level.INFO, className, methodName, "Accessing the CustomerLocal in the iterator");

                        //to find the accounts count 
                        int count = 0;
                        CustomerKey cKey = new CustomerKey(c.getPrimaryKey().hashCode());
                        CustomerLocal customer = customerHome.findByPrimaryKey(cKey);
                        count = customer.getAccountsCount();

                        //putting all the data into a bean
                        CustomerData data = new CustomerData(c.getPrimaryKey().hashCode(), c.getFirstName(), c.getLastName(), count, c.getTaxID());
                        list.add(data);
                    }

                    pgBean.setListByLastName(list);

                } catch (FinderException e) {
                    errmsg.add(bundle.getString("succLastNameSearch"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("succLastNameSearch") + e.getMessage());
                } catch (Exception e) {
                    errmsg.add(bundle.getString("errGenFailureSearch"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errGenFailureSearch") + e.getMessage());
                    e.printStackTrace();
                }
            } //end of the if

            if (pgBean.getListByLastName() == null || pgBean.getListByLastName().size() == 0) {
                msg.add(bundle.getString("succLastNameSearch"));
            }
            pgBean.setErrormessage(errmsg);
            pgBean.setListByLastName(list);
            pgBean.setMessageLines(msg);

            req.setAttribute("PageBean", pgBean);
            session.setAttribute("HistoryBean", history);// stores it back into the session

            if (pgBean.getErrormessage().size() == 0 && pgBean.getListByLastName().size() > 0) {
                getServletContext().getRequestDispatcher("/jsp/listofcustomers.jsp").forward(req, resp);
            } else {
                getServletContext().getRequestDispatcher("/jsp/searchbycustomer.jsp").forward(req, resp);
            }

        } catch (Throwable theException) {
            // uncomment the following line when unexpected exceptions are occuring to aid in debugging the problem
            // theException.printStackTrace();
            throw new ServletException(theException);
        }
    }
}
