//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

// General EJB.
import java.io.IOException;
import java.io.Serializable;
import java.util.ResourceBundle;

import javax.ejb.FinderException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.TransferLocal;
import com.ibm.websphere.samples.bank.ejb.TransferLocalHome;
import com.ibm.websphere.samples.bank.exception.InsufficientFundsException;

import java.util.logging.*;

/**
 * Demonstration servlet showing a session EJB accessing an entity EJB
 * to get account balances or to transfer funds between accounts.
 **/
public class TransferFunds extends HttpServlet implements Serializable {

    private ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.websphere.samples.bank.web.ClientResourceBundle");

    // Strings for HTML output - US English defaults shown.
    static String amount = "Amount";
    static String from = "From Account";
    static String to = "To Account";
    static String successtransfer = "Transfer successful";
    static String successbalance = "Balance query successful";
    static String failureinsufficient = "Insufficient funds - no transfer";
    static String failurerequest = "Internal failure - could not do request";
    static String failureinput = "Bad input - could not do request";
    static String failureinternal = "Internal failure - could not create account:";
    static String failureamount = "Invalid amount format:";
    static String failureaccount = "Invalid account format:";
    static String failurefind = "One (or both) accounts not found";
    static String failuregenexp = "Exception:";

    // EJB access.
    private TransferLocalHome transferHome = null;
    private final static String TRANSFER_JNDI_NAME = "java:comp/env/ejb/Transfer";

    static final String className = TransferFunds.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
     * Read in internationalization strings when servlet is loaded. <p>
     * Create TransferHome when servlet is first loaded. <p>
     * @param config Configuration information passed by the servlet engine.
     * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
     */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);
        // Get national language text for HTML response.
        try {
            // Get NLS strings from an external resource bundle identified by
            // "com.ibm.websphere.samples.bank.web.ClientResourceBundle", to override
            // US English defaults.
            amount = bundle.getString("amount");
            from = bundle.getString("from");
            to = bundle.getString("to");
            successtransfer = bundle.getString("successtransfer");
            successbalance = bundle.getString("successbalance");
            failureinsufficient = bundle.getString("failureinsufficient");
            failurerequest = bundle.getString("failurerequest");
            failureinput = bundle.getString("failureinput");
            failureinternal = bundle.getString("failureinternal");
            failureamount = bundle.getString("failureamount");
            failureaccount = bundle.getString("failureaccount");
            failurefind = bundle.getString("failurefind");
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, "init resource bundle exception: " + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }

        try {
            Context ctx = new InitialContext();
            Object homeObject = ctx.lookup(TRANSFER_JNDI_NAME);

            transferHome = (TransferLocalHome) homeObject;

        }
        // Determine approximate cause of failure.
        catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, "init NamingException: " + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, "init Exception: " + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }

    public void doPost(HttpServletRequest req, HttpServletResponse res) throws IOException, ServletException {
        performTask(req, res);
    }

    public void doGet(HttpServletRequest req, HttpServletResponse res) throws IOException, ServletException {
        performTask(req, res);
    }

    /**
     * Respond to user request to transfer funds between accounts, or get an account balance. <p>
     * @param req Used to get information about the user request - an account number, etc.
     * @param res Used to pass response back to the user - an output stream for the returned HTML, etc.
     * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
     * @exception java.io.IOException Signals that an I/O exception of some sort has occurred.
     */
    public void performTask(HttpServletRequest req, HttpServletResponse res) throws ServletException {
        final String methodName = "performTask";

        // --- Read and validate user input, initialize. ---
        // Error flags.
        boolean fromAccountFlag = true;
        boolean toAccountFlag = true;
        boolean amountFlag = true;
        boolean inputFlag = false;
        boolean transferFlag = false;
        boolean insufficientFlag = false;
        boolean finderFlag = false;

        // Datatypes used in Transfer object methods.
        float amountFloat = 0;
        int fromKey = 0;
        int toKey = 0;

        // Used in creating return HTML.
        String fromBalance = "***";
        String toBalance = "***";
        String messageLine1 = "";
        String messageLine2 = "";
        String fromAccount = "";
        String toAccount = "";
        String accountBeanMode = "";

        try {
            // Was transfer, fromgetbalance, or togetbalance button selected?
            String[] transferArray = req.getParameterValues("transfer");
            String[] fromArray = req.getParameterValues("fromgetbalance");
            String[] toArray = req.getParameterValues("togetbalance");
            int request = 0;

            if (transferArray != null && transferArray.length > 0)
                request = 1; // transfer between accounts.
            else if (fromArray != null && fromArray.length > 0)
                request = 2; // get a balance for "from" account.
            else if (toArray != null && toArray.length > 0)
                request = 3; // get a balance for "to" account.

            // Get user keyed in parameters...  some may be null.
            String[] fromAccountArray = req.getParameterValues("fromaccount");
            String[] toAccountArray = req.getParameterValues("toaccount");
            String[] amountArray = req.getParameterValues("amount");
            accountBeanMode = req.getParameter("beanMode");

            // Validate the user input.
            if (request == 1) { // need all three user inputs
                // from account
                try {
                    fromKey = Integer.parseInt(fromAccountArray[0]);
                    fromAccount = "" + fromKey;
                } catch (Exception e) {
                    fromAccountFlag = false;
                }
                // to account
                try {
                    toKey = Integer.parseInt(toAccountArray[0]);
                    toAccount = "" + toKey;
                } catch (Exception e) {
                    toAccountFlag = false;
                }
                // amount to transfer
                try {
                    amountFloat = (Float.valueOf(amountArray[0])).floatValue();
                } catch (Exception e) {
                    amountFlag = false;
                }
            } else if (request == 2) { // just need account for balance query
                // fromaccount balance
                try {
                    fromKey = Integer.parseInt(fromAccountArray[0]);
                    fromAccount = "" + fromKey;
                } catch (Exception e) {
                    fromAccountFlag = false;
                }
            } else if (request == 3) { // just need account for balance query
                // toaccount balance
                try {
                    toKey = Integer.parseInt(toAccountArray[0]);
                    toAccount = "" + toKey;
                } catch (Exception e) {
                    fromAccountFlag = false;
                    fromAccountArray[0] = toAccountArray[0];
                }
            }

            // --- If input parameters are good, try to transfer or get balance. ---
            if (amountFlag && fromAccountFlag && toAccountFlag) {
                inputFlag = true; // Client input is good if we get here.
                TransferLocal transfer = null;

                try {
                    // Create a Transfer object.
                    transfer = transferHome.create();
                    // Based on user request type, invoke appropriate transfer methods.
                    if (request == 1) {
                        transfer.transferFunds(fromKey, toKey, amountFloat);
                        fromBalance = "" + formatFloat(transfer.getBalance(fromKey));
                        toBalance = "" + formatFloat(transfer.getBalance(toKey));
                        transferFlag = true;
                    } else if (request == 2) {
                        fromBalance = "" + formatFloat(transfer.getBalance(fromKey));
                        transferFlag = true;
                    } else if (request == 3) {
                        toBalance = "" + formatFloat(transfer.getBalance(toKey));
                        transferFlag = true;
                    }
                }
                // Determine approximate cause of failure.
                catch (InsufficientFundsException e) {
                    insufficientFlag = true;
                    messageLogger.logp(Level.INFO, className, methodName, "InsufficientFundsException: " + e.getMessage());
                    e.printStackTrace();
                } catch (FinderException e) {
                    finderFlag = true;
                    messageLogger.logp(Level.INFO, className, methodName, "FinderException: " + e.getMessage());
                    e.printStackTrace();
                } catch (Exception e) {
                    messageLogger.logp(Level.INFO, className, methodName, "Exception: " + e.getMessage());
                    e.printStackTrace();
                }
            }

            // --- Prepare messages to accompany response. ---
            if (inputFlag) {
                // The client input was OK - did transfer methods work?
                if (transferFlag) {
                    // Transfer methods worked
                    if (request == 1)
                        messageLine1 = successtransfer;
                    else
                        messageLine1 = successbalance;
                } else if (insufficientFlag) {
                    // Not enough money in "from" account.
                    messageLine1 = failureinsufficient;
                    messageLine2 = amount + ": " + amountFloat + ", " + from + ": " + fromKey + ", " + to + ": " + toKey;
                } else if (finderFlag) {
                    // One (or both) accounts not found.
                    messageLine1 = failurefind;
                    messageLine2 = from + ": " + fromKey + ", " + to + ": " + toKey;
                } else {
                    // Other reason for failure (might be non-existent account)
                    messageLine1 = failureinternal;
                }
            } else {
                // Something was wrong with the client input
                String separator = "";
                if (!amountFlag) {
                    messageLine1 = failureinput;
                    messageLine2 = failureamount + " " + amountArray[0];
                    separator = ", ";
                }
                if (!fromAccountFlag) {
                    messageLine1 = failureinput;
                    messageLine2 = messageLine2 + separator + failureaccount + " " + fromAccountArray[0];
                    separator = ", ";
                }
                if (!toAccountFlag) {
                    messageLine1 = failureinput;
                    messageLine2 = messageLine2 + separator + failureaccount + " " + toAccountArray[0];
                }
            }

            JSPBean jspbean = new JSPBean();
            jspbean.setFromAccount(fromAccount);
            jspbean.setToAccount(toAccount);
            jspbean.setFromBalance(fromBalance);
            jspbean.setToBalance(toBalance);
            jspbean.setAccountBeanMode(accountBeanMode);
            jspbean.setMessageLine1(messageLine1);
            jspbean.setMessageLine2(messageLine2);

            javax.servlet.http.HttpSession session = req.getSession(true);
            session.setAttribute("JSPBean", jspbean);
            RequestDispatcher rd = getServletContext().getRequestDispatcher("TransferFundsJSP.jsp");
            rd.forward(req, res);
        } catch (Throwable theException) {
            // uncomment the following line when unexpected exceptions are occuring to aid in debugging the problem
            // theException.printStackTrace();
            throw new ServletException(theException);
        }

    }

    /**
     * @param x
     * @return String representation of float
     */
    private String formatFloat(float x) {
        String s = null;
        if (((x * 100) % 10) == 0) {
            s = x + "0";
        } else {
            s = String.valueOf(x);
        }
        return s;
    }

}
